# Adaptation Michael Dolores

# This function can be used to output information to the image_client
# log file if the script is not working correctly.
#
proc log_error { msg } {

  # Uncomment the following line to log errors
  puts error $msg
}

#
# This function will examine the information for
# and image and determine the correct series description
# for a series containing only this image.
#
# param series [IN] : The series to examine.
#
# return : The series description to be used.
#
proc determine_series_description { series } {

  # Initialize constants
  set GENERATED_SERIES_DESCRIPTION "generated_series_description"

  # Get generated series description and use it directly - don't bother trying to read both
  # hanging_protocol and series_description etc, just use the generated value.
  set series_description [ get_info $GENERATED_SERIES_DESCRIPTION $series ]
  return $series_description

}



# 
# This function will take a list of series and generated new series
# with only one image per series.  Only series of type MG will be split,
# all others will be ignored.
#
# param study [IN] : The study to which the list of series belongs
# param series_list [IN] : The list of series to be split
#
# return : The number of series created.
#
proc split_series { study series_list } {

  # Initialized constants
  set SERIES_DESCRIPTION "series_description"
  set SERIES_MODALITY "series_modality"
  set SERIES_REF "series_ref"
  set OBJECT_REF "image_object_ref"
  set num_new_series 0
  
  # Ensure that there are items in the list.
  if { [llength $series_list ] <= 0 } {
    return $num_new_series
  }

  # Now we split!
  #
  foreach series $series_list {
	
    set series_modality [ get_info $SERIES_MODALITY $series ]
    set series_description [ get_info $SERIES_DESCRIPTION $series ]

    # For this script, only MG series will be split.
    #
    if { $series_description != "EG_10 chos_ STEATOSE+FER" } {
      continue
    }
	
    set image_list [ get_images $series ]
    set num_images [ llength $image_list ]

    # The first image will remain in the current series, while the
    # rest of the image will be placed in their own series.
    #
    if { $num_images <= 1 } {
      continue
    }
	  
    variable new_series_list ""
    set last_series_copied $series

    for { set i 0 } { $i < [expr ($num_images - 1)] } { incr i } {

	# Copy the series.
	# The new series will be placed after the series being copied.
	# This is not a deep copy, so images are not copied.
	#
	set cloned_series [children clone $study $last_series_copied]
      	lappend new_series_list $cloned_series
      
	# The original series remains the source for all images.
	# Images are removed one at a time and added to the empty cloned series.
	#
      	set image_to_move [lindex $image_list [expr ($i + 1)]]
      	children remove $series $image_to_move
      	children add_to_tail $cloned_series $image_to_move

	# To ensure consistency within the series
	# a new series_ref will be assigned.
	#
        set image_ref [ get_info $OBJECT_REF $image_to_move ]
        set series_ref [expr ($image_ref * -1)]

	put_info $SERIES_REF $cloned_series $series_ref

	# To maintain series order while we clone, update last_series_copied
	#
      	set last_series_copied $cloned_series
	
	incr num_new_series
    }
	    
  }

  return $num_new_series 
}



# 
# This function will take a list of series and combine the series
# that share a common description.  
#
# param study [IN] : The study to which the list of series belongs
# param series_list [IN] : The list of series to be combined
#
# return : The number of series remaining.
#
proc regroup_series { study series_list } {
 
  set new_series_list ""
  set SERIES_MODALITY "series_modality"
  set SERIES_DESCRIPTION "series_description"
  log_error "Starting regroup...."

  # This function will grab the first (base) MG series and search for matches
  # in the other series in the study.  A series is considered a match 
  # if the series description is the same.  All the images from a matched
  # series are appended to the base series, leaving the matched series empty
  # and removable.
  #

  log_error "1"

  while {[llength $series_list] > 0} {
	
    # Use two lists to ensure we do not check for a match on one item more than once
    #	 
    lappend new_series_list [lindex $series_list 0]
    set series_list [lreplace $series_list 0 0]
    
    set series_to_match [lindex $new_series_list [expr [llength $new_series_list] -1]]
    set series_to_match_modality [ get_info $SERIES_MODALITY $series_to_match ]
    set series_to_match_description [ get_info $SERIES_DESCRIPTION $series_to_match ]

    # This series to match must be MG

    while { $series_to_match_description != "EG_10 chos_ STEATOSE+FER" && [llength $series_list] > 0 } {
	    
	# Clear out the first appended item
	#
	set new_series_list [lreplace $new_series_list 0 0]

        lappend new_series_list [lindex $series_list 0]
        set series_list [lreplace $series_list 0 0]
    
        set series_to_match [lindex $new_series_list [expr [llength $new_series_list] -1]]
        set series_to_match_modality [ get_info $SERIES_MODALITY $series_to_match ]
        set series_to_match_description [ get_info $SERIES_DESCRIPTION $series_to_match ]
    }
    
  log_error "2"

    if { [llength $series_list] <= 0 } {
	return 0
    }
    
    
    set series_to_match_description [ determine_series_description $series_to_match ]

    set length [ string length $series_to_match_description ]

    if { $length == 0 } {
	continue
    }

  log_error "3"

    log_error "Series to match description ... $series_to_match_description"

    # Compare the last $new_series_list against the remaining $series_list
    #
    foreach series_to_check $series_list {
log_error "3a"
    set series_modality [ get_info $SERIES_MODALITY $series_to_check ]
    set series_description [ get_info $SERIES_DESCRIPTION $series_to_check ]
log_error "3b"
    # For this script, only MG series will be split.
    #
    if { $series_description != "EG_10 chos_ STEATOSE+FER" } {
      continue
    }

  log_error "4"

      set series_to_check_description [ determine_series_description $series_to_check ]

      log_error "Series to check description ... $series_to_check_description"

      # Check to see if the compared attribute string in NULL
      if { $series_to_match_description == $series_to_check_description } {
	  log_error "Match on $series_to_match_description!!!!"    
    	  set images_to_move [get_images $series_to_check]

	  foreach image $images_to_move {    
     	    children remove $series_to_check $image
	    children add_to_tail $series_to_match $image
      	    put_info $SERIES_DESCRIPTION $series_to_match $series_to_match_description
    	  }

	  # Remove series from study
	  #
	  children remove $study $series_to_check
	  
    	  # Move the matched series out of the $series_list list
	  set pos_series_to_check [lsearch -exact $series_list $series_to_check ]
	  linsert new_series_list 0 $pos_series_to_check
	  set remove_match_from_list $pos_series_to_check
    	  set series_list [lreplace $series_list $remove_match_from_list $remove_match_from_list]
      }

      children sort_descending $series_to_match [lindex [get_images $series_to_match] 0]
    }
  }
 
  return [llength $new_series_list]
}




# Use the selected study to get the list of series
# that need to be split.
#
set study_list [get_studies selected] 
set first_study [lindex $study_list 0]
set series_list [get_series $first_study]

set number_of_new_series [ split_series $first_study $series_list ]

# Destroy invalid object handles and set new handles
#
destroy_token all


# With the images now split, the common images can be combined.
#
set study_list [get_studies selected]
set first_study [lindex $study_list 0]
set series_list [get_series $first_study]

set actual_number_of_series [ regroup_series $first_study $series_list ]

destroy_token all